==> datahub-restful-core-1.1.8.tgz: package/routes/entities-router.js <==
'use strict';

const debug = require('debug')('krh:route');
const route = require('koa-router');

/**
 * Entity path route,export entities(tables)
 * @type {Router|exports|module.exports}
 */

const router = new route();
const joinConditions = {
    join:{},
    innerJoin:{},
    leftJoin:{},
    rightJoin:{},
    leftOuterJoin:{},
    rightOuterJoin:{},
    fullOuterJoin:{},
    crossJoin:{}
};

router
    .all('/:entity(.*)', function* (next) {

        //entity path resolver
        let entity = this.params.entity;

        if (entity.search('.') > -1) {
            entity = entity.split('.')[0];
        }

        if (!entity) {
            this.body = '缺少实体类参数';
            return this.status = 404;
        }

        /*let isEntityExists = yield this.state.connection.schema
            .withSchema(this.state.schema)
            .hasTable(entity);

        //Detect table existing.
        if (!isEntityExists) {
            this.body = '表' + entity + '不存在';
            return this.status = 404;
        }*/

        //多表
        if(entity === 'join'){

            let conditionsQS = this.query._joinOn || '';
            let entity = this.query._joinFrom;
            delete this.query['_joinFrom'];
            delete this.query['_joinOn'];

            this.state.entity = entity;

            let conditions  = conditionsQS instanceof Array ? conditionsQS : conditionsQS.split(';');
            if(!conditionsQS){
                this.throw('invalid join _joinOn query string, can not be empty', 400);
            }
            this.state.joinConditions = conditions;

            /*
            if(!joinConditions[this.query._joinConditions]){

                this.throw('invalid join condition query string:' + this.query._joinCondition
                    + '\nThe query string [_joinCondition] should be one of these:'
                    + Object.keys(joinConditions).join(',')
                    + '\nBy default is join.', 400);
            }else{
                this.state.joinCondition = this.query._joinCondition || 'join';
                this.body = 'join';
            }*/

        }else{
            this.state.entity = entity;
        }

        if (this.get('async')) {
            this.state.async = true;
        }

        //Call restful chain
        yield next;
    })
    .get('/', function* () {

        yield this.state.connection.schema
            .raw('use :schema:', {schema: this.state.schema});

        //Read and export table meta data and href
        let tables = yield this.state.connection.schema
            .raw('show tables in :schema:', {schema: this.state.schema});

        debug('Has tables', tables);

        let host = [this.protocol, this.header['x-datahub-host'] || this.request.host].join('://');

        //[1],why 1? See knex.schema.raw, show tables command is the second promise
        //And why [1][0]? 0 represents the result of "show tables".
        this.body = tables[0].map(e => {

            var tableNameKey = ['Tables', 'in', this.state.schema].join('_');
            var tableName = e[tableNameKey];

            debug(e, tableName, tableNameKey);
            var path = [
                this.request.path,
                tableName].join('/').replace(/\/\//g, '/');
            return {
                table: tableName,
                href: host + path
            }
        });
    })
    .get('/raw', function* (next) {

        //Raw sql execution
        let sql = this.query.sql;
        delete this.query.sql;

        this.body = yield this.query.length == 0
            ? this.state.connection.raw(sql) : this.state.connection.raw(sql, this.query.sql);
    })
    .get('/_ddl', function* (next) {

        //Read and export table meta data and href
        let tables = yield this.state.connection.schema
            .raw('show tables in :schema:', {schema: this.state.schema});

        debug('Has tables', tables);

        let res = yield Promise.all(tables[0].map(e => {
            var tableNameKey = ['Tables', 'in', this.state.schema].join('_');
            var tableName = e[tableNameKey];

            return this.state.connection.raw("show create table " + `${this.state.schema}.${tableName}`);
        }));

        this.body = res.map(e => e[0][0]['Create Table']).join(';\n\n');
    });

module.exports = router;

==> datahub-restful-core-2.0.3.tgz: package/routes/entities-router.js <==
'use strict';

const debug = require('debug')('krh:route');
const route = require('koa-router');

/**
 * Entity path route,export entities(tables)
 * @type {Router|exports|module.exports}
 */

const router = new route();
const joinConditions = {
    join:{},
    innerJoin:{},
    leftJoin:{},
    rightJoin:{},
    leftOuterJoin:{},
    rightOuterJoin:{},
    fullOuterJoin:{},
    crossJoin:{}
};

router
    .all('/:entity(.*)', function* (next) {

        //entity path resolver
        let entity = this.params.entity;

        if (entity.search('.') > -1) {
            entity = entity.split('.')[0];
        }

        if (!entity) {
            this.body = '缺少实体类参数';
            return this.status = 404;
        }

        /*let isEntityExists = yield this.state.connection.schema
            .withSchema(this.state.schema)
            .hasTable(entity);

        //Detect table existing.
        if (!isEntityExists) {
            this.body = '表' + entity + '不存在';
            return this.status = 404;
        }*/

        //多表
        if(entity === 'join'){

            let conditionsQS = this.query._joinOn || '';
            let entity = this.query._joinFrom;
            delete this.query['_joinFrom'];
            delete this.query['_joinOn'];

            // 检查权限
            if (!this.privilegeHelper.canAccessTable(this.state.database, this.state.schema, entity)) {
                this.throw(403, '无权限')
                return;
            }

            this.state.entity = entity;

            let conditions = conditionsQS instanceof Array ? conditionsQS : conditionsQS.split(';');

            if(!conditionsQS){
                this.throw('invalid join _joinOn query string, can not be empty', 400);
            }

            // 如果 join 的表中有一个没有权限
            if (conditions.some(entity => !this.privilegeHelper.canAccessTable(this.state.database, this.state.schema, entity))) {
                this.throw(403, '无权限')
                return;
            }

            this.state.joinConditions = conditions;
        }else{
            // 检查权限
            if (!this.privilegeHelper.canAccessTable(this.state.database, this.state.schema, entity)) {
                this.throw(403, '无权限')
                return;
            }

            this.state.entity = entity;
        }

        if (this.get('async')) {
            this.state.async = true;
        }

        //Call restful chain
        yield next;
    })
    .get('/', function* () {

        yield this.state.connection.schema
            .raw('use :schema:', {schema: this.state.schema});

        //Read and export table meta data and href
        let tables = yield this.state.connection.schema
            .raw('show tables in :schema:', {schema: this.state.schema});

        debug('Has tables', tables);

        let host = [this.protocol, this.header['x-datahub-host'] || this.request.host].join('://');

        //[1],why 1? See knex.schema.raw, show tables command is the second promise
        //And why [1][0]? 0 represents the result of "show tables".
        this.body = tables[0]
            .map(e => {
                var tableNameKey = ['Tables', 'in', this.state.schema].join('_');
                var tableName = e[tableNameKey];

                debug(e, tableName, tableNameKey);
                var path = [
                    this.request.path,
                    tableName].join('/').replace(/\/\//g, '/');
                return {
                    table: tableName,
                    href: host + path
                }
            })
            .filter(e => this.privilegeHelper.canAccessTable(this.state.database, this.state.schema, e.table));
    })
    .get('/raw', function* (next) {

        //Raw sql execution
        let sql = this.query.sql;
        delete this.query.sql;

        this.body = yield this.query.length == 0
            ? this.state.connection.raw(sql) : this.state.connection.raw(sql, this.query.sql);
    })
    .get('/_ddl', function* (next) {

        //Read and export table meta data and href
        let tables = yield this.state.connection.schema
            .raw('show tables in :schema:', {schema: this.state.schema});

        debug('Has tables', tables);

        let res = yield Promise.all(tables[0]
            .map(e => {
                var tableNameKey = ['Tables', 'in', this.state.schema].join('_');
                var tableName = e[tableNameKey];
                return tableName;
            })
            .filter(tableName => this.privilegeHelper.canAccessTable(this.state.database, this.state.schema, tableName))
            .map(tableName => {
                return this.state.connection.raw("show create table " + `${this.state.schema}.${tableName}`);
            })
        );

        this.body = res.map(e => e[0][0]['Create Table']).join(';\n\n');
    });

module.exports = router;
